'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.selectRow = exports.filterRows = exports.filterRow = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _reselect = require('reselect');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint no-console: 0 */

var checkInput = function checkInput(selector, key, value) {
  if (!_lodash2.default.isFunction(selector)) {
    throw new Error('`selector` should be a function.');
  }

  if (!_lodash2.default.isString(key)) {
    throw new Error('`key` should be a string.');
  }

  if (_lodash2.default.isUndefined(value)) {
    throw new Error('`value` should be defined.');
  }
};

/**
 * Filter the results of `selector` and return an object.
 * If several items match, it throws an error.
 *
 * @param  {Function}         Selector function (`createSelector` from reselect)
 * @param  {String}           Key
 * @param  {String | Number}  Value
 * @return {Function}         Selector
 */
var filterRow = exports.filterRow = function filterRow(selector, key, value) {
  checkInput(selector, key, value);

  return (0, _reselect.createSelector)(selector, function (rows) {
    var results = _lodash2.default.groupBy(rows, key)[value];
    var row = undefined;

    if (_lodash2.default.isUndefined(results)) {
      row = {};
    } else if (results.length === 1) {
      row = results.pop();
    } else if (!results.length) {
      row = {};
    } else {
      throw new Error('Your selector should not return more than 1 row.');
    }

    return row;
  });
};

/**
 * Filter the results of `selector` and return an array.
 * If several items match, it throws an error.
 *
 * @param  {Function}         Selector function (`createSelector` from reselect)
 * @param  {String}           Key
 * @param  {String | Number}  Value
 * @return {Function}         Selector
 */
var filterRows = exports.filterRows = function filterRows(selector, key, value) {
  checkInput(selector, key, value);

  return (0, _reselect.createSelector)(selector, function (rows) {
    return _lodash2.default.groupBy(rows, key)[value] || [];
  });
};

/**
 * DEPRECATED
 *
 * Select one or severals rows from `selector` if the key `key` is equal to
 * `value`.
 *
 * @param  {Function}         Selector function (`createSelector` from reselect)
 * @param  {String}           Key
 * @param  {String | Number}  Value
 * @return {Function}         Selector
 */
var selectRow = exports.selectRow = function selectRow(selector, key, value) {
  console.info('\n`selectRow` is deprecated. Please use `filterRows` instead.\nNotice that `filterRows` doesn\'t call the selector by default, like \\createSelector` from `reselect` does.\n');

  if (_lodash2.default.isUndefined(selector) || _lodash2.default.isUndefined(key) || _lodash2.default.isUndefined(value)) {
    return function () {
      return 'Bad arguments for `selectRow`';
    };
  }

  return (0, _reselect.createSelector)(selector(), function (rows) {
    var result = _lodash2.default.groupBy(rows, key)[value];
    if (_lodash2.default.isArray(result) && result.length === 1) {
      result = result.pop();
    }
    return result;
  });
};